using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SampleClient.PTBurnTestingCtrl
{
    public partial class PTBurnTestingCtrl : UserControl
    {
        public Robot CurrentRobot;
        private StatusFile MyStatusFile;
        private string Server;
        private string StatusFile;
        private string LogFile;
        
        public PTBurnTestingCtrl()
        {
            InitializeComponent();
        }

        public void SetRobot(Robot currentRobotIn, string serverIn)
        {
            if (currentRobotIn == null)
            {
                this.buttonStartTest.Enabled = false;
            }
            else
            {
                this.buttonStartTest.Enabled = true;
                CurrentRobot = currentRobotIn;
                this.Server = serverIn;
                this.StatusFile = "\\\\" + this.Server + "\\PTBurnJobs\\Status\\" + CurrentRobot.StatusFile;
                this.LogFile = "\\\\" + this.Server + "\\PTBurnJobs\\Log\\PTLog.txt";
                MyStatusFile = new StatusFile(this.StatusFile);
            }
        }

        private void buttonStartTest_Click(object sender, EventArgs e)
        {
            FormRunTest myTestRunner = new FormRunTest();

            DialogResult dRst = MessageBox.Show("Running this test deletes the log and status files.  Are you sure you want to continue?", "Warning", MessageBoxButtons.YesNo);
            if (dRst == DialogResult.Yes)
            {
                for (int i = 0; i < this.listViewTestJobs.CheckedItems.Count; i++)
                {
                    myTestRunner.JobsToRun.Add(this.listViewTestJobs.CheckedItems[i].SubItems[0].Text);
                }
                myTestRunner.RunIndependently = checkBoxRunJobsIndependently.Checked;
                myTestRunner.nIterations = (UInt32)numericUpDownTestIterations.Value;
                myTestRunner.sJobDir = Properties.Settings.Default.TestJobLocations[comboBoxLocations.SelectedIndex];
                myTestRunner.sServer = this.Server;
                myTestRunner.sReportDir = textBoxTestReport.Text;
                myTestRunner.sStatusFile = StatusFile;
                myTestRunner.sLogFile = LogFile;
                myTestRunner.nAbortRate = (UInt32)numericUpDownAbortRate.Value;
                myTestRunner.Show();
            }
        }

        private void UpdateTestJobList()
        {
            string[] items;
            this.listViewTestJobs.Items.Clear();

            DirectoryInfo myJobsDir = new DirectoryInfo(Properties.Settings.Default.TestJobLocations[comboBoxLocations.SelectedIndex]);
            if (!myJobsDir.Exists)
                Directory.CreateDirectory(myJobsDir.FullName);

            FileInfo[] myFiles = myJobsDir.GetFiles();
            foreach (FileInfo myFile in myJobsDir.GetFiles())
            {
                JobFile myJob = new JobFile();
                if (myJob.LoadJobFile(myFile.FullName))
                {
                    if (String.IsNullOrEmpty(myJob.Robot))
                        items = new string[] { myFile.Name, myJob.Copies.ToString(), "Any" };
                    else
                        items = new string[] { myFile.Name, myJob.Copies.ToString(), myJob.Robot };

                    ListViewItem myItem = new ListViewItem(items);
                    myItem.Checked = true;
                    this.listViewTestJobs.Items.Add(myItem);
                }
                else
                    MessageBox.Show("Unable to load job file");
            }

        }

        private void PTBurnTestingCtrl_Load(object sender, EventArgs e)
        {
            
            this.textBoxTestReport.Text = Properties.Settings.Default.TestResultPath;
            foreach (string sPath in Properties.Settings.Default.TestJobLocations)
            {
                this.comboBoxLocations.Items.Add(sPath);

            }

            //this.textBoxJobDir.Text = Properties.Settings.Default.TestJobPath;
            comboBoxLocations.SelectedIndex = 0;
            this.numericUpDownTestIterations.Value = 1;

            UpdateTestJobList();

            DirectoryInfo myJobsDir = new DirectoryInfo(this.comboBoxLocations.Items[comboBoxLocations.SelectedIndex].ToString());
            this.fileSystemWatcher1.Path = myJobsDir.FullName;
            this.fileSystemWatcher1.Filter = "*.jrq";
            //this.textBoxJobDir.Text = myJobsDir.FullName;


        }

        private void textBoxJobDir_TextChanged(object sender, EventArgs e)
        {

        }

        private void textBoxTestReport_TextChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.TestResultPath = textBoxTestReport.Text;
            Properties.Settings.Default.Save();
        }

        private void fileSystemWatcher1_Created(object sender, FileSystemEventArgs e)
        {
            UpdateTestJobList();
        }

        private void fileSystemWatcher1_Deleted(object sender, FileSystemEventArgs e)
        {
            UpdateTestJobList();
        }

        private void checkBoxUncheckAll_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void buttonUncheckAll_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem myItem in this.listViewTestJobs.Items)
            {
                myItem.Checked = false;

            }
        }

        private void buttonCheckAll_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem myItem in this.listViewTestJobs.Items)
            {
                myItem.Checked = true;

            }
        }

        private void comboBoxLocations_SelectedIndexChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.TestJobPath = Properties.Settings.Default.TestJobLocations[comboBoxLocations.SelectedIndex];
            Properties.Settings.Default.Save();
            UpdateTestJobList();
        }
    }
}
