using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PrimeraTechnology;
using System.Threading;

namespace PTRobotUserControl
{  
    public partial class FormRunTest : Form
    {
        private ThreadSettings testSettings;
        public FormRunTest(ThreadSettings testSettingsIn)
        {
            InitializeComponent();
            testSettings = testSettingsIn;
        }

        private static PTRobotReturn WaitForCommand(uint nRobotID)
        {
            PTRobotReturn retVal = PTRobotReturn.Ok;
            RobotStatus myRobotStatus = new RobotStatus();

            while (true)
            {
                retVal =PTRobot.GetRobotStatus(nRobotID,ref myRobotStatus);
                if (PTRobotReturn.Ok == retVal)
                {
                    if (myRobotStatus.SystemState == SystemState.Idle)
                        break;
                    else if (myRobotStatus.SystemState == SystemState.Error)
                    {

                    }
                }
                Thread.Sleep(500);


            }
            return retVal;
        }

        private void backgroundWorkerTestRunner_DoWork(object sender, DoWorkEventArgs e)
        {
            ThreadSettings testSettings = e.Argument as ThreadSettings;
            RobotInfo myRobotInfo = new RobotInfo();
            UInt32 driveID;
            
            for (int i = 0; i < testSettings.Copies; i++)
            {
                int nPercent = (i / (int)testSettings.Copies) * 100;
                if (backgroundWorkerTestRunner.CancellationPending)
                    break;
                PTRobot.GetRobotInfo(testSettings.RobotID, ref myRobotInfo);
                if (testSettings.UseRecorders)
                {
                    if (myRobotInfo.NumDrives > 1 && (i % 2 != 0))
                        driveID = testSettings.DriveArray[1];
                    else
                        driveID = testSettings.DriveArray[0];

                    backgroundWorkerTestRunner.ReportProgress(nPercent, "Loading Drive");
                    PTRobot.LoadDrive(testSettings.RobotID, driveID, testSettings.FromLocation, ClearDrive.Yes);
                    WaitForCommand(testSettings.RobotID);

                    if (backgroundWorkerTestRunner.CancellationPending)
                        break;

                    if (testSettings.UsePrinter)
                    {
                        backgroundWorkerTestRunner.ReportProgress(nPercent, "Loading Printer from Drive");
                        PTRobot.LoadPrinterFromDrive(testSettings.RobotID, driveID);
                        WaitForCommand(testSettings.RobotID);

                        if (backgroundWorkerTestRunner.CancellationPending)
                            break;

                        if (!String.IsNullOrEmpty(testSettings.PrintFile))
                        {
                            backgroundWorkerTestRunner.ReportProgress(nPercent, "Printing"+ testSettings.PrintFile);
                            PTRobot.PrintFile(testSettings.RobotID, testSettings.PrintFile, (uint)i);
                            WaitForCommand(testSettings.RobotID);
                        }

                        if (backgroundWorkerTestRunner.CancellationPending)
                            break;

                        backgroundWorkerTestRunner.ReportProgress(nPercent, "Unloading Printer");
                        PTRobot.UnLoadPrinter(testSettings.RobotID, testSettings.ToLocation);
                        WaitForCommand(testSettings.RobotID);

                    }
                    else
                    {

                        if (backgroundWorkerTestRunner.CancellationPending)
                            break;
                        
                        backgroundWorkerTestRunner.ReportProgress(nPercent, "Unloading Drive");
                        PTRobot.UnLoadDrive(testSettings.RobotID, driveID, testSettings.ToLocation);
                        WaitForCommand(testSettings.RobotID);
                    }
                }
                else
                {
                    if (testSettings.UsePrinter)
                    {
                        backgroundWorkerTestRunner.ReportProgress(nPercent, "Loading Printer");
                        PTRobot.LoadPrinter(testSettings.RobotID, testSettings.FromLocation);
                        WaitForCommand(testSettings.RobotID);

                        if (backgroundWorkerTestRunner.CancellationPending)
                            break;
                        
                        if (!String.IsNullOrEmpty(testSettings.PrintFile))
                        {
                            backgroundWorkerTestRunner.ReportProgress(nPercent, "Printing" + testSettings.PrintFile);
                            PTRobot.PrintFile(testSettings.RobotID, testSettings.PrintFile, (uint)i);
                            WaitForCommand(testSettings.RobotID);
                        }

                        if (backgroundWorkerTestRunner.CancellationPending)
                            break;
                        
                        backgroundWorkerTestRunner.ReportProgress(nPercent, "Unloading Printer");
                        PTRobot.UnLoadPrinter(testSettings.RobotID, testSettings.ToLocation);
                        WaitForCommand(testSettings.RobotID);
                    }
                    else
                    {
                        PTRobot.MoveDiscBetweenLocations(testSettings.RobotID, testSettings.FromLocation, testSettings.ToLocation);
                        WaitForCommand(testSettings.RobotID);
                    }
                }
            }
            
            backgroundWorkerTestRunner.ReportProgress(100, "Finished");
            System.Threading.Thread.Sleep(3000);
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void FormRunTest_Load(object sender, EventArgs e)
        {
            this.backgroundWorkerTestRunner.RunWorkerAsync(this.testSettings);
        }

        private void FormRunTest_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.backgroundWorkerTestRunner.CancelAsync();
        }

        private void backgroundWorkerTestRunner_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.labelProgress.Text = e.UserState.ToString();
            this.progressBarOverall.Value = e.ProgressPercentage;
        }

        private void backgroundWorkerTestRunner_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.Close();
        }
    }

    public class ThreadSettings
    {
        public uint RobotID = 0;
        public uint[] DriveArray = new uint[10];
        public string PrintFile = null;
        public uint Copies = 0;
        public PrimeraTechnology.DiscLocation ToLocation;
        public PrimeraTechnology.DiscLocation FromLocation;
        public bool UseRecorders = false;
        public bool UsePrinter = false;
        public string MergeData1 = "";
        public string MergeData2 = "";
        public string MergeData3 = "";
        public string MergeData4 = "";
        public string sMergeData5 = "";
    }
    
}