using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;

namespace SampleClient
{
    public enum SystemState {};
    public enum JobState  {JOB_NOT_STARTED=0, JOB_RUNNING, JOB_COMPLETED_SUCCESSFULLY, JOB_FAILED};
    public enum LoadDiscState  {DISC_LOADED=1, DISC_PROCESSING, DISC_PROCESSED, DISC_UNLOADING};
    public enum DriveState {IDLE=0, RECORDING, READING,VERIFYING, DISC_LOADED, VEFIFY_FAILED, VERIFY_COMPLETE, RECORD_FAILED, RECORD_COMPLETE};
    public enum SystemError {NO_ERRORS=0, PRINTER_MOVE, INVALID_CARTRIDGE, INPUT_EMPTY, INTERNAL_COMM, COLOR_LOW, BLACK_LOW, BOTH_LOW, PICK, ARM_MOVEMENT,
    PRINTER_MOVEMENT, ADMIN, INTERNAL_SOFTWARE, NO_ROBOTIC_DRIVES, OFFLINE, COVER_OPEN, PICK2, DROPPED_DISC_RECORDER, DROPPED_DISC_PRINTER, DROPPED_DISC_BIN,
    DROPPED_DISC_FRONT, CARTRIDGE_ALIGNMENT, INVALID_COLOR_CARTRIDGE, INVALID_BLACK_CARTRIDGE, BOTH_CARTRIDGES_INVALID, NO_CARTRIDGES_INSTALLED, BLACK_IN_COLOR,
    COLOR_IN_BLACK, COLOR_BLACK_SWAPPED, PIGMENT_CARTRIDGE, ALIGNMENT_FAILED};

    public enum JobError {NO_ERRORS=0, INTERNAL_RECORDING_ENGINE, INTERNAL_JOB_PROCESSOR, INTERAL_ROBOTICS, FILE_INVALID, TOO_MANY_FILES, NO_FILES, PRINT_FILE_INVALID,
    JOB_FILE_INVALID, STATUS_FILE_INVALID, MEDIA_INVALID, MEDIA_TOO_SMALL, MEDIA_NOT_BLANK, DRIVE_OPENCLOSE, DRIVE_NOT_READY, DRIVE_NOT_ROBOTIC, JOB_ABORTED, DVD_INVALID,
    RECORDING_ERROR, VERIFY_ERROR, REJECTS_EXCEEDED, SESSION_INVALID, CLIENT_INVALID, CLIENT_MESSAGE_ERROR, UNKNOWN_JOB_TYPE, CRITICAL_KEY_INVAID, TEMP_FOLDER_OVERFLOW,
    CDTEXT_INVAILD, PRINT_APP_NOT_INSTALLED, PRINT_FILE_MISSING, INVALID_CARTRIDGES_FOR_PRINT, READ_LOCATION_TOO_SMALL, READING_ERROR, PVD_INVALID, INVALID_JOB_FOR_PVD};

    class Drive
    {
        private string DriveSection;
        private string File;
        public int Index;


        public string Description;
        public string Location;
        public UInt32 GoodDiscs;
        public UInt32 BadDiscs;
        public UInt32 ReadErrors;
        public UInt32 ReadErrorsMedia;
        public UInt32 RecordingErrors;
        public UInt32 RecordingErrorsMedia;
        public UInt32 VerifyErrors;
        public UInt32 VerifyErrorsMedia;
        public DriveState State;
        public UInt32 Percent;
        public string letter;
        public string Job;
        public UInt32 Disc;
        public UInt32 DriveNotReadyErrors;

        public Drive(int IndexIn, string FileIn)
        {
            File = FileIn;
            Index = IndexIn;
            Refresh();
        }

        public void Refresh()
        {
            string Temp;
            //Get the drive info keys
            Temp = Utility.INIGetKeyValue("System", "DriveDesc" +Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
                this.Description = Temp;
            Temp = Utility.INIGetKeyValue("System", "DriveLocation" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
                this.Location = Temp;
            DriveSection = this.Location;
            Temp = Utility.INIGetKeyValue("System", "DriveState" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
                this.State = (DriveState) Convert.ToUInt32(Temp);
            Temp = Utility.INIGetKeyValue("System", "DrivePercent" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
            {
                if (Temp == "None")
                    this.Percent = 0;
                else
                    this.Percent = Convert.ToUInt32(Temp);
            }

            Temp = Utility.INIGetKeyValue("System", "DriveLetter" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
                this.letter = Temp;

            Temp = Utility.INIGetKeyValue("System", "DriveJob" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
                this.Job = Temp;
            Temp = Utility.INIGetKeyValue("System", "DriveDisc" + Index.ToString(), File);
            if (!String.IsNullOrEmpty(Temp))
            {
                if (Temp == "None")
                    this.Disc = 0;
                else
                    this.Disc = Convert.ToUInt32(Temp);
            }

            //Get the drive statistics section
            Temp = Utility.INIGetKeyValue(DriveSection, "GoodDiscs", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.GoodDiscs = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.GoodDiscs = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "BadDiscs", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.BadDiscs = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.BadDiscs = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "ReadingErrors", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.ReadErrors = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.ReadErrors = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "ReadingErrorsMedia", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.ReadErrorsMedia = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.ReadErrorsMedia = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "RecordingErrors", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.RecordingErrors = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.RecordingErrors = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "RecordingErrorsMedia", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.RecordingErrorsMedia = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.RecordingErrorsMedia = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "VerifyingErrors", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.VerifyErrors = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.VerifyErrors = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "VerifyingErrorsMedia", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.VerifyErrorsMedia = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.VerifyErrorsMedia = 99;
                }
            }

            Temp = Utility.INIGetKeyValue(DriveSection, "DriveNotReady", File);
            if (!String.IsNullOrEmpty(Temp))
            {
                try
                {
                    this.DriveNotReadyErrors = Convert.ToUInt32(Temp);
                }
                catch
                {
                    this.DriveNotReadyErrors = 99;
                }
            }

        }

    }

    class Job
    {
        private string JobSection;
        private string File;
        

        public UInt32 JobsAhead;
        public UInt32 DiscsAhead;
        public string Status;
        public UInt32 GoodDiscs;
        public UInt32 BadDiscs;
        public UInt32 DiscsRemaining;
        public string TimeStarted;
        public string TimeCompleted;
        public string TimeSubmitted;
        public JobState JobState;
        public string Name;
        public JobError JobError;
        public string JobErrorString;
        public string Client;
        public bool CompletedJob;
        
        public Job(string JobSectionIn, string FileIn, bool CompletedJobIn)
        {
            JobSection = Name = JobSectionIn;
            File = FileIn;
            CompletedJob = CompletedJobIn;
            this.Refresh();
        }
        public void Refresh()
        {
            string Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "JobsAhead", File);
            if (!String.IsNullOrEmpty(Temp))
                this.JobsAhead = Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "DiscsAhead", File);
            if (!String.IsNullOrEmpty(Temp))
                this.DiscsAhead = Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "CurrentStatus", File);
            if (!String.IsNullOrEmpty(Temp))
                this.Status = Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "GoodDiscs", File);
            if (!String.IsNullOrEmpty(Temp))
                this.GoodDiscs = Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "BadDiscs", File);
            if (!String.IsNullOrEmpty(Temp))
                this.BadDiscs = Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "DiscsRemaining", File);
            if (!String.IsNullOrEmpty(Temp))
                this.DiscsRemaining = Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "JobState", File);
            if (!String.IsNullOrEmpty(Temp))
                this.JobState = (JobState)Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "JobErrorNumber", File);
            if (!String.IsNullOrEmpty(Temp))
                this.JobError = (JobError)Convert.ToUInt32(Temp);

            Temp = Utility.INIGetKeyValue(JobSection, "JobErrorString", File);
            if (!String.IsNullOrEmpty(Temp))
                this.JobErrorString = Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "ClientID", File);
            if (!String.IsNullOrEmpty(Temp))
                this.Client = Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "TimeCompleted", File);
            if (!String.IsNullOrEmpty(Temp))
                this.TimeCompleted = Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "TimeStarted", File);
            if (!String.IsNullOrEmpty(Temp))
                this.TimeStarted = Temp;

            Temp = Utility.INIGetKeyValue(JobSection, "TimeSubmitted", File);
            if (!String.IsNullOrEmpty(Temp))
                this.TimeSubmitted = Temp;

        }
    }
    
    
    
    class StatusFile
    {
        public ArrayList Jobs;
        public ArrayList Drives;
        private string File;
        public string RobotFW;
        public string SerialNum;
        public string ManfDate;
        public string SystemDesc;
        public string SystemStatus;
        public string ServerTime;
        public SystemError SystemError;
        public string SystemErrorString;
        public UInt32 DriveSpace;

        
        
        public StatusFile(string fileIn)
        {
            Jobs = new ArrayList();
            Drives = new ArrayList();
            
            File = fileIn;
            Refresh();
        }

        public bool IsJobOnList(string job, ref Job returnedJob)
        {
            bool retVal = false;
            returnedJob = null;
            
            foreach (Job myJob in this.Jobs)
            {
                if (myJob.Name == job)
                {
                    retVal = true;
                    returnedJob = myJob;
                    break;
                }
            }
            return retVal;
        }

        public void Refresh()
        {
            string temp;
            temp = Utility.INIGetKeyValue("System", "RoboFWVer", File);
            if (!String.IsNullOrEmpty(temp))
                this.RobotFW = temp;

            temp = Utility.INIGetKeyValue("System", "SerialNum", File);
            if (!String.IsNullOrEmpty(temp))
            this.SerialNum = temp;

            temp = Utility.INIGetKeyValue("System", "DateManf", File);
            if (!String.IsNullOrEmpty(temp))
            this.ManfDate = temp;

            temp = Utility.INIGetKeyValue("System", "SystemDesc", File);
            if (!String.IsNullOrEmpty(temp))
            this.SystemDesc = temp;

            temp = Utility.INIGetKeyValue("System", "SystemStatus", File);
            if (!String.IsNullOrEmpty(temp))
            this.SystemStatus = temp;

            temp = Utility.INIGetKeyValue("System", "ServerTime", File);
            if (!String.IsNullOrEmpty(temp))
            this.ServerTime = temp;

            temp = Utility.INIGetKeyValue("System", "SysErrorNumber", File);
            if (!String.IsNullOrEmpty(temp))
            this.SystemError = (SystemError)Convert.ToUInt32(temp);

            temp = Utility.INIGetKeyValue("System", "SysErrorString", File);
            if (!String.IsNullOrEmpty(temp))
            this.SystemErrorString = temp;
            
            RefreshJobs();
            RefreshDrives();
        }

        public void RefreshDrives()
        {
            int drive = 0;
            this.Drives.Clear();
            while (true)
            {
                string sDriveKey = "DriveLocation" + drive.ToString();
                string sTemp = Utility.INIGetKeyValue("System", sDriveKey, File);
                if (!String.IsNullOrEmpty(sTemp))
                    this.Drives.Add(new Drive(drive, File));
                else
                    break;
                drive++;
            }
        }

        public void RefreshJobs()
        {
            int job = 0;
            this.Jobs.Clear();
            while (true)
            {
                string sJobKey = "Job" + job.ToString();
                string sTemp = Utility.INIGetKeyValue("CompletedJobs", sJobKey, File);
                if (!String.IsNullOrEmpty(sTemp))
                    this.Jobs.Add(new Job(sTemp, File, true));
                else
                    break;
                job++;
            }
            job = 0;
            while (true)
            {
                string sJobKey = "Job" + job.ToString();
                string sTemp = Utility.INIGetKeyValue("JobList", sJobKey, File);
                if (!String.IsNullOrEmpty(sTemp))
                    this.Jobs.Add(new Job(sTemp, File, false));
                else
                    break;
                job++;
            }
        }
    }
}
